@@ -187,10 +187,14 @@ fut_header (unsigned long code, ... )
 #ifdef __MINGW32__
 #ifdef __i386__
   #define ma_cmpxchg(ptr,o,n) \
-	InterlockedCompareExchange((ptr),(n),(o))
+	InterlockedCompareExchange((LONG volatile *)(ptr),(LONG)(n),(LONG)(o))
+  #define add_return(ptr,s) \
+	(InterlockedExchangeAdd((ptr),(s))+(s))
 #elif defined (__x86_64__)
-  #define ma_cmpxchg(ptr,o,n) \
-	InterlockedCompareExchange64((ptr),(n),(o))
+#define ma_cmpxchg(ptr,o,n) \
+	InterlockedCompareExchange64((LONGLONG volatile *)(ptr),(LONGLONG)(n),(LONGLONG)(o))
+  #define add_return(ptr,s) \
+	(InterlockedExchangeAdd64((ptr),(s))+(s))
 #else
 #error TODO test the long size
 #endif
@@ -207,10 +211,14 @@ fut_header (unsigned long code, ... )
 #define x86_64_sync() __asm__ __volatile__("lock; addl $0,0(%%rsp)" ::: "memory")
 #define ppc_sync() __asm__ __volatile__("sync" ::: "memory")
 
-#if defined(__i386__) || defined(__KNC__) || defined(__KNF__)
+#if defined(__i386__)
 #define mb() i386_sync()
 #define rmb() i386_sync()
 #define wmb() i386_sync()
+#elif defined(__KNC__) || defined(__KNF__)
+#define mb() x86_64_sync()
+#define rmb() mb()
+#define wmb() mb()
 #elif defined(__x86_64__)
 #define mb() x86_64_sync()
 #define rmb() __asm__ __volatile__("lfence" ::: "memory")
@@ -229,9 +237,11 @@ static unsigned long trash_buffer[8];
 extern int allow_fut_flush;
 extern pthread_mutex_t fut_flush_lock;
 
+#if !defined(__MINGW32__)
 #ifdef __GNUC__
 __attribute__((weak))
 #endif
+#endif
 uint64_t fut_getstamp(void)
 {
 	unsigned long long tick;
@@ -244,6 +254,10 @@ uint64_t fut_getstamp(void)
 	clock_gettime(CLOCK_MONOTONIC, &tp);
 #  endif
 	tick = 1000000000ULL*tp.tv_sec+ tp.tv_nsec;
+#elif defined(__MINGW32__)
+	struct timeval tv;
+	gettimeofday(&tv, NULL);
+	tick = 1000000ULL * tv.tv_usec + tv.tv_usec;
 #else
 	TBX_GET_TICK(tick);
 #endif
