@@ -94,7 +94,7 @@ int record_tid_activated = 0;
  */
 int allow_fut_flush = 0;
 
-pthread_mutex_t fut_flush_lock;
+pthread_mutex_t fut_flush_lock = PTHREAD_MUTEX_INITIALIZER;
 
 static int already_flushed = 0;
 
