@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004, 2011 Samuel Thibault <samuel.thibault@ens-lyon.org>
+ *  Copyright (C) 2004, 2011, 2013 Samuel Thibault <samuel.thibault@ens-lyon.org>
  *
  * This program is free software ; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -78,10 +78,22 @@ struct fxt_infos {
 #define FXT_ARCH_I386 0
 #define FXT_ARCH_IA64 1
 #define FXT_ARCH_X86_64 2
+#define FXT_ARCH_ALPHA 3
+#define FXT_ARCH_ARM32 4
+#define FXT_ARCH_ARM64 5
+#define FXT_ARCH_MIPSEL 6
+#define FXT_ARCH_SH 7
 
 /* Big-endian machines */
 #define FXT_ARCH_PPC 0x10000
 #define FXT_ARCH_PPC64 0x20000
+#define FXT_ARCH_AVR32 0x30000
+#define FXT_ARCH_HPPA 0x40000
+#define FXT_ARCH_M68K 0x50000
+#define FXT_ARCH_MIPS 0x60000
+#define FXT_ARCH_S390 0x70000
+#define FXT_ARCH_S390X 0x80000
+#define FXT_ARCH_SPARC 0x90000
 	uint32_t arch;     /* so that syscalls/irqs/... are known */
 
 	unsigned int ncpus;
@@ -326,48 +338,73 @@ struct fxt_trace_user_raw64 {
         uint64_t args[];
 };
 
-#undef FXT_ARCH_64
-#undef FXT_ARCH_32
 #ifdef __i386
 #  define FXT_RECORD_ARCH FXT_ARCH_I386
-#  define FXT_BLOCK_TRACES_KERNEL_RAW FXT_BLOCK_TRACES_KERNEL_RAW32
-#  define FXT_BLOCK_TRACES_USER_RAW FXT_BLOCK_TRACES_USER_RAW32
-#  define FXT_BLOCK_TRACES_USER_RAW_NATIVE FXT_BLOCK_TRACES_USER_RAW32
-#  define FXT_ARCH_32
-typedef struct fxt_trace_user_raw32 fxt_trace_user_raw_t;
-typedef struct fxt_trace_kernel_raw32 fxt_trace_kernel_raw_t;
+#define FXT_ARCH_32
 
 #elif defined __ia64
-#  define FXT_ARCH_64
-#  define FXT_RECORD_ARCH FXT_ARCH_IA64;
-#  define FXT_BLOCK_TRACES_KERNEL_RAW FXT_BLOCK_TRACES_KERNEL_RAW64
-#  define FXT_BLOCK_TRACES_USER_RAW FXT_BLOCK_TRACES_USER_RAW64
-#  define FXT_BLOCK_TRACES_USER_RAW_NATIVE FXT_BLOCK_TRACES_USER_RAW64
-typedef struct fxt_trace_user_raw64 fxt_trace_user_raw_t;
+#  define FXT_RECORD_ARCH FXT_ARCH_IA64
+#define FXT_ARCH_64
 
 #elif defined __x86_64__
-#  define FXT_RECORD_ARCH FXT_ARCH_X86_64;
-#  define FXT_BLOCK_TRACES_KERNEL_RAW FXT_BLOCK_TRACES_KERNEL_RAW64
-#  define FXT_BLOCK_TRACES_USER_RAW FXT_BLOCK_TRACES_USER_RAW64
-#  define FXT_BLOCK_TRACES_USER_RAW_NATIVE FXT_BLOCK_TRACES_USER_RAW64
-#  define FXT_ARCH_64
-typedef struct fxt_trace_user_raw64 fxt_trace_user_raw_t;
+#  define FXT_RECORD_ARCH FXT_ARCH_X86_64
+#define FXT_ARCH_64
+
+#elif defined(__alpha__) || defined(__alpha)
+#  define FXT_RECORD_ARCH FXT_ARCH_ALPHA
+#define FXT_ARCH_64
+
+#elif defined(__arm__)
+#  define FXT_RECORD_ARCH FXT_ARCH_ARM32
+#define FXT_ARCH_32
+
+#elif defined(__aarch64__)
+#  define FXT_RECORD_ARCH FXT_ARCH_ARM64
+#define FXT_ARCH_64
+
+#elif defined(__AVR__) || defined(__AVR)
+#  define FXT_RECORD_ARCH FXT_ARCH_AVR32
+#define FXT_ARCH_32
+
+#elif defined(__hppa__) || defined(__hppa)
+#  define FXT_RECORD_ARCH FXT_ARCH_HPPA
+#define FXT_ARCH_32
+
+#elif defined(__m68k__)
+#  define FXT_RECORD_ARCH FXT_ARCH_M68K
+#define FXT_ARCH_32
+
+#elif defined(__MIPSEL__)
+#  define FXT_RECORD_ARCH FXT_ARCH_MIPSEL
+#define FXT_ARCH_32
+
+#elif defined(__mips__) || defined(__mips)
+#  define FXT_RECORD_ARCH FXT_ARCH_MIPS
+#define FXT_ARCH_32
 
 #elif defined(__powerpc__) || defined(__ppc__) || defined(_ARCH_PPC)
-#  define FXT_RECORD_ARCH FXT_ARCH_PPC;
-#  define FXT_BLOCK_TRACES_KERNEL_RAW FXT_BLOCK_TRACES_KERNEL_RAW32
-#  define FXT_BLOCK_TRACES_USER_RAW FXT_BLOCK_TRACES_USER_RAW32
-#  define FXT_BLOCK_TRACES_USER_RAW_NATIVE FXT_BLOCK_TRACES_USER_RAW32
-#  define FXT_ARCH_32
-typedef struct fxt_trace_user_raw32 fxt_trace_user_raw_t;
+#  define FXT_RECORD_ARCH FXT_ARCH_PPC
+#define FXT_ARCH_32
 
 #elif defined(__PPC64__) || defined(__powerpc64__)
-#  define FXT_RECORD_ARCH FXT_ARCH_PPC64;
-#  define FXT_BLOCK_TRACES_KERNEL_RAW FXT_BLOCK_TRACES_KERNEL_RAW64
-#  define FXT_BLOCK_TRACES_USER_RAW FXT_BLOCK_TRACES_USER_RAW64
-#  define FXT_BLOCK_TRACES_USER_RAW_NATIVE FXT_BLOCK_TRACES_USER_RAW64
-#  define FXT_ARCH_64
-typedef struct fxt_trace_user_raw64 fxt_trace_user_raw_t;
+#  define FXT_RECORD_ARCH FXT_ARCH_PPC64
+#define FXT_ARCH_64
+
+#elif defined(__s390x__)
+#  define FXT_RECORD_ARCH FXT_ARCH_S390X
+#define FXT_ARCH_64
+
+#elif defined(__s390__)
+#  define FXT_RECORD_ARCH FXT_ARCH_S390
+#define FXT_ARCH_32
+
+#elif defined(__sparc__) || defined(__sparc)
+#  define FXT_RECORD_ARCH FXT_ARCH_SPARC
+#define FXT_ARCH_32
+
+#elif defined(__sh__)
+#  define FXT_RECORD_ARCH FXT_ARCH_SH
+#define FXT_ARCH_32
 
 #else
 #  error Unable to find a supported host architecture.
@@ -376,9 +413,18 @@ typedef struct fxt_trace_user_raw64 fxt_trace_user_raw_t;
 #ifdef FXT_ARCH_32
 #  define FXT_TRACE_KERNEL_RAW FXT_TRACE_KERNEL_RAW32
 #  define FXT_TRACE_USER_RAW FXT_TRACE_USER_RAW32
+#  define FXT_BLOCK_TRACES_KERNEL_RAW FXT_BLOCK_TRACES_KERNEL_RAW32
+#  define FXT_BLOCK_TRACES_USER_RAW FXT_BLOCK_TRACES_USER_RAW32
+#  define FXT_BLOCK_TRACES_USER_RAW_NATIVE FXT_BLOCK_TRACES_USER_RAW32
+typedef struct fxt_trace_user_raw32 fxt_trace_user_raw_t;
+typedef struct fxt_trace_kernel_raw32 fxt_trace_kernel_raw_t;
 #else /* FXT_ARCH_64 */
 #  define FXT_TRACE_KERNEL_RAW FXT_TRACE_KERNEL_RAW64
 #  define FXT_TRACE_USER_RAW FXT_TRACE_USER_RAW64
+#  define FXT_BLOCK_TRACES_KERNEL_RAW FXT_BLOCK_TRACES_KERNEL_RAW64
+#  define FXT_BLOCK_TRACES_USER_RAW FXT_BLOCK_TRACES_USER_RAW64
+#  define FXT_BLOCK_TRACES_USER_RAW_NATIVE FXT_BLOCK_TRACES_USER_RAW64
+typedef struct fxt_trace_user_raw64 fxt_trace_user_raw_t;
 #endif
 
 #endif /* __FXT_H */
