@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2003, 2004, 2011-2012  Samuel Thibault <samuel.thibault@ens-lyon.org>
+ *  Copyright (C) 2003, 2004, 2011-2012, 2016  Samuel Thibault <samuel.thibault@ens-lyon.org>
  *
  * This program is free software ; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -94,8 +94,10 @@ int fkt_record_pids( fxt_t fxt, int fd )
 
 	if( !(dir = opendir("/proc")) )
 		{
+#ifndef __MINGW32__
 		perror("opening /proc");
 		fprintf(stderr,"pids won't be recorded\n");
+#endif
 		}
     else {
 		while( (dirent = readdir(dir)) )
